#include "Fixed.hpp"
#include <iostream>
#include <cmath>
#include <ostream>

Fixed::~Fixed()
{
	std::cout << "Destructor called" << std::endl;
}

Fixed::Fixed()
{
	this->_value = 0;
	std::cout << "Default constructor called" << std::endl;
}

Fixed::Fixed(const Fixed& src)
{
	std::cout << "Copy constructor called" << std::endl;
	this->_value = src._value;
}

Fixed::Fixed(const int& src)
{
	std::cout << "Int constructor called" << std::endl;
	this->_value = src << this->_nb_bits;
}

Fixed::Fixed(const float& src)
{
	std::cout << "Float constructor called" << std::endl;
	this->_value = (int) roundf(src * (1 << this->_nb_bits));
}

Fixed& Fixed::operator=(const Fixed& src)
{
	std::cout << "Copy assignment operator called" << std::endl;
	this->_value = src._value;
	return (*this);
}

int Fixed::getRawBits() const
{
	std::cout << "getRawBits member function called" << std::endl;
	return (this->_value);
}

void Fixed::setRawBits(const int raw)
{
	std::cout << "setRawBits member function called" << std::endl;
	this->_value = raw;
}

float Fixed::toFloat(void) const
{
	return ((float) this->_value / (1 << this->_nb_bits));
}

int Fixed::toInt() const
{
	return ((int) (roundf(this->toFloat())));
}

std::ostream& operator<<(std::ostream &out, const Fixed& fixed)
{
	out << (fixed.toFloat());
	return (out);
}

Fixed& Fixed::operator++()
{
	this->_value++;
	return (*this);
}

Fixed& Fixed::operator--()
{
	this->_value--;
	return (*this);
}

Fixed Fixed::operator++(int)
{
	Fixed tmp(*this);
	this->_value++;
	return (tmp);
}

Fixed Fixed::operator--(int)
{
	Fixed tmp(*this);
	this->_value--;
	return (tmp);
}

bool Fixed::operator==(const Fixed& src) const
{
	return (this->_value == src._value);
}

bool Fixed::operator!=(const Fixed& src) const
{
	return (this->_value != src._value);
}

bool Fixed::operator>=(const Fixed& src) const
{
	return (this->_value >= src._value);
}

bool Fixed::operator<=(const Fixed& src) const
{
	return (this->_value <= src._value);
}

bool Fixed::operator>(const Fixed& src) const
{
	return (this->_value > src._value);
}

bool Fixed::operator<(const Fixed& src) const
{
	return (this->_value < src._value);
}

Fixed Fixed::operator*(const Fixed &src) const
{
	return Fixed(this->toFloat() * src.toFloat());
}

Fixed Fixed::operator/(const Fixed &src) const
{
	return Fixed(this->toFloat() / src.toFloat());
}

Fixed Fixed::operator+(const Fixed &src) const
{
	return Fixed(this->toFloat() + src.toFloat());
}

Fixed Fixed::operator-(const Fixed &src) const
{
	return Fixed(this->toFloat() - src.toFloat());
}

const Fixed& Fixed::max(const Fixed& f1, const Fixed &f2)
{
	if (f1._value > f2._value)
		return (f1);
	return (f2);
}

Fixed& Fixed::max(Fixed& f1, Fixed &f2)
{
	if (f1._value > f2._value)
		return (f1);
	return (f2);
}

const Fixed& Fixed::min(const Fixed& f1, const Fixed &f2)
{
	if (f1._value < f2._value)
		return (f1);
	return (f2);
}

Fixed& Fixed::min(Fixed& f1, Fixed &f2)
{
	if (f1._value < f2._value)
		return (f1);
	return (f2);
}
